package net.wanji.web.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import net.wanji.feign.pojo.result.JsonViewObject;
import net.wanji.feign.pojo.vo.ControlCommandVO;
import net.wanji.feign.pojo.vo.CrossIdVO;
import net.wanji.feign.pojo.vo.LockControlVO;
import net.wanji.feign.pojo.vo.ScheduleSendVO;
import net.wanji.feign.pojo.vo.SchemeSendVO;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.web.dto.CrossIdAndPhaseIdsDTO;
import net.wanji.web.dto.CrossIdAndTimeDTO;
import net.wanji.web.dto.CrossIdDTO;
import net.wanji.web.dto.RunningPlanDTO;
import net.wanji.web.dto.SaveSchemeConfigDTO;
import net.wanji.web.dto.ScheduleIdDTO;
import net.wanji.web.dto.SchemeIdDTO;
import net.wanji.web.mapper.scheme.CrossLightsMapper;
import net.wanji.web.mapper.scheme.CrossPhaseLightsMapper;
import net.wanji.web.mapper.scheme.CrossPhaseMapper;
import net.wanji.web.mapper.scheme.CrossPlanMapper;
import net.wanji.web.mapper.scheme.CrossSchedulesPlanMapper;
import net.wanji.web.mapper.scheme.CrossSchemeMapper;
import net.wanji.web.po.scheme.CrossLightsPO;
import net.wanji.web.po.scheme.CrossPhaseLightsPO;
import net.wanji.web.po.scheme.CrossPhasePO;
import net.wanji.web.po.scheme.CrossPlanPO;
import net.wanji.web.po.scheme.CrossSchedulesPlanPO;
import net.wanji.web.po.scheme.CrossSchemePO;
import net.wanji.web.service.PlanSendService;
import net.wanji.web.service.scheme.impl.RunningPlanServiceImpl;
import net.wanji.web.service.scheme.impl.SchemeConfigServiceImpl;
import net.wanji.web.vo.PhaseListByTimeVO;
import net.wanji.web.vo.SchemePhaseLightsVO;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/2/1 9:39
 */
@Service
public class PlanSendServiceImpl implements PlanSendService {
    private final UtcFeignClients utcFeignClients;
    private final CrossSchedulesPlanMapper crossSchedulesPlanMapper;
    private final CrossPlanMapper crossPlanMapper;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;
    private final CrossLightsMapper crossLightsMapper;
    private final CrossPhaseLightsMapper crossPhaseLightsMapper;
    private final RunningPlanServiceImpl runningPlanServiceImpl;
    private final SchemeConfigServiceImpl schemeConfigServiceImpl;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
    SimpleDateFormat hourMinuteSdf = new SimpleDateFormat("HH:mm");

    private Gson gson = new GsonBuilder().setDateFormat("yyyy-MM-dd").create();

    public PlanSendServiceImpl(@Qualifier("net.wanji.feign.service.UtcFeignClients") UtcFeignClients utcFeignClients, CrossSchedulesPlanMapper crossSchedulesPlanMapper, CrossPlanMapper crossPlanMapper, CrossSchemeMapper crossSchemeMapper, CrossPhaseMapper crossPhaseMapper, CrossLightsMapper crossLightsMapper, CrossPhaseLightsMapper crossPhaseLightsMapper, RunningPlanServiceImpl runningPlanServiceImpl, SchemeConfigServiceImpl schemeConfigServiceImpl) {
        this.utcFeignClients = utcFeignClients;
        this.crossSchedulesPlanMapper = crossSchedulesPlanMapper;
        this.crossPlanMapper = crossPlanMapper;
        this.crossSchemeMapper = crossSchemeMapper;
        this.crossPhaseMapper = crossPhaseMapper;
        this.crossLightsMapper = crossLightsMapper;
        this.crossPhaseLightsMapper = crossPhaseLightsMapper;
        this.runningPlanServiceImpl = runningPlanServiceImpl;
        this.schemeConfigServiceImpl = schemeConfigServiceImpl;
    }

    @Override
    public JsonViewObject scheduleSend(ScheduleIdDTO scheduleIdDTO) {
        String crossId = scheduleIdDTO.getCrossId();
        ScheduleSendVO scheduleSendVO = new ScheduleSendVO();
        scheduleSendVO.setCrossCode(crossId);
        // 构造时间表
        List<ScheduleSendVO.Schedule> schedules = buildSchedules(crossId, scheduleIdDTO);
        scheduleSendVO.setSchedules(schedules);
        // 发送请求
        JsonViewObject jsonViewObject = utcFeignClients.scheduleSend(scheduleSendVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-时间表下发UTC服务调用异常");
        }
        return jsonViewObject.success();
    }

    @Override
    public JsonViewObject schemeSend(SchemeIdDTO schemeIdDTO) {
        String crossId = schemeIdDTO.getCrossId();
        SchemeSendVO schemeSendVO = new SchemeSendVO();
        schemeSendVO.setCrossCode(crossId);
        // 构造方案列表
        Integer schemeId = schemeIdDTO.getSchemeId();
        List<SchemeSendVO.Pattern> patternList = buildPatternList(crossId, schemeId);
        schemeSendVO.setPatternList(patternList);
        // 发送请求
        JsonViewObject jsonViewObject = utcFeignClients.schemeSend(schemeSendVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-基础方案下发UTC服务调用异常");
        }
        return jsonViewObject.success();
    }

    @Override
    public JsonViewObject nextStep(CrossIdDTO crossIdDTO) {
        String crossId = crossIdDTO.getCrossId();
        // command 1开始步进，stepNum 0顺序步进
        JsonViewObject jsonViewObject = utcFeignClients.stepControl(crossId, 1, 0);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-步进控制UTC服务调用异常");
        }
        return jsonViewObject.success();
    }

    @Override
    public JsonViewObject lockPhase(CrossIdAndPhaseIdsDTO crossIdAndPhaseIdsDTO) {
        LockControlVO lockControlVO = new LockControlVO();
        lockControlVO.setCrossCode(crossIdAndPhaseIdsDTO.getCrossId());
        lockControlVO.setCommand(1); // 控制标识：1是；0否
        lockControlVO.setDuration(0); // 长期

        List<Integer> phaseIds = crossIdAndPhaseIdsDTO.getPhaseIds();
        List<Integer> phaseList = new ArrayList<>();
        for (Integer phaseId : phaseIds) {
            CrossPhasePO crossPhasePO = crossPhaseMapper.selectById(phaseId);
            String phaseNo = crossPhasePO.getPhaseNo();
            phaseList.add(Integer.parseInt(phaseNo));
        }
        lockControlVO.setPhaseList(phaseList);

        JsonViewObject jsonViewObject = utcFeignClients.lockControl(lockControlVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-锁定控制UTC服务调用异常");
        }
        return jsonViewObject.success();
    }

    @Override
    public JsonViewObject allRedControl(CrossIdDTO crossIdDTO) {
        ControlCommandVO commandVO = getControlCommandVO(crossIdDTO);
        commandVO.setControlType(1); //1全红 2黄闪 3关灯
        JsonViewObject jsonViewObject = utcFeignClients.allRedControl(commandVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-全红控制UTC服务调用异常");
        }
        return jsonViewObject.success();
    }

    @Override
    public JsonViewObject yellowLightControl(CrossIdDTO crossIdDTO) {
        ControlCommandVO commandVO = getControlCommandVO(crossIdDTO);
        commandVO.setControlType(2); //1全红 2黄闪 3关灯
        JsonViewObject jsonViewObject = utcFeignClients.yellowLightControl(commandVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-黄闪控制UTC服务调用异常");
        }
        return jsonViewObject.success();
    }

    @Override
    public JsonViewObject restore(CrossIdAndPhaseIdsDTO crossIdAndPhaseIdsDTO) {
        String crossId = crossIdAndPhaseIdsDTO.getCrossId();
        List<Integer> phaseIds = crossIdAndPhaseIdsDTO.getPhaseIds();
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        // 取消步进
        // command 1开始步进，stepNum 0顺序步进
        jsonViewObject = utcFeignClients.stepControl(crossId, 0, 0);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-步进控制UTC服务调用异常");
        }

        // 取消相位锁定
        LockControlVO lockControlVO = new LockControlVO();
        lockControlVO.setCrossCode(crossIdAndPhaseIdsDTO.getCrossId());
        lockControlVO.setCommand(0); // 控制标识：1是；0否
        lockControlVO.setDuration(0); // 长期
        List<Integer> phaseList = new ArrayList<>();
        for (Integer phaseId : phaseIds) {
            CrossPhasePO crossPhasePO = crossPhaseMapper.selectById(phaseId);
            String phaseNo = crossPhasePO.getPhaseNo();
            phaseList.add(Integer.parseInt(phaseNo));
        }
        lockControlVO.setPhaseList(phaseList);

        jsonViewObject = utcFeignClients.lockControl(lockControlVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-锁定控制UTC服务调用异常");
        }

        // 取消全红控制
        CrossIdDTO crossIdDTO = new CrossIdDTO();
        crossIdDTO.setCrossId(crossId);
        ControlCommandVO commandVO = getControlCommandVO(crossIdDTO);
        commandVO.setCommand(0); //控制标识：1是；0否
        commandVO.setControlType(1); //1全红 2黄闪 3关灯
        jsonViewObject = utcFeignClients.allRedControl(commandVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-全红控制UTC服务调用异常");
        }

        // 取消黄闪控制
        commandVO.setControlType(2); //1全红 2黄闪 3关灯
        jsonViewObject = utcFeignClients.yellowLightControl(commandVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-黄闪控制UTC服务调用异常");
        }
        return jsonViewObject.success();
    }

    @Override
    @Transactional
    public JsonViewObject syncScheme(CrossIdDTO crossIdDTO) {
        CrossIdVO crossIdVO = new CrossIdVO();
        crossIdVO.setCrossId(crossIdDTO.getCrossId());
        JsonViewObject jsonViewObject = utcFeignClients.schemePhaseLights(crossIdVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机静态信息获取-方案信息-UTC服务调用异常");
        }
        Object content = jsonViewObject.getContent();
        if (ObjectUtil.isNotNull(content)) {
            // 信息入库
            String contentStr = gson.toJson(content);
            SchemePhaseLightsVO schemePhaseLightsVO = gson.fromJson(contentStr, SchemePhaseLightsVO.class);
            if (ObjectUtil.isEmpty(schemePhaseLightsVO)) {
                return jsonViewObject.fail("信号机方案对象转换结果为空，未更新数据库");
            }
            List<CrossSchemePO> crossSchemeList = schemePhaseLightsVO.getCrossSchemeList();
            for (CrossSchemePO crossSchemePO : crossSchemeList) {
                String crossId = crossSchemePO.getCrossId();
                String schemeNo = crossSchemePO.getSchemeNo();
                Integer schemeId = crossSchemeMapper.selectIdByCrossIdAndSchemeNo(crossId, schemeNo);
                if (!ObjectUtil.isEmpty(schemeId)) {
                    // 如果有相同的路口ID和方案号，则更新
                    crossSchemeMapper.update(crossSchemePO);
                } else {
                    // 否则插入
                    crossSchemeMapper.insertOne(crossSchemePO);
                }
            }
            List<CrossPhasePO> crossPhaseList = schemePhaseLightsVO.getCrossPhaseList();
            for (CrossPhasePO crossPhasePO : crossPhaseList) {
                String crossId = crossPhasePO.getCrossId();
                String phaseNo = crossPhasePO.getPhaseNo();
                Integer phaseId = crossPhaseMapper.selectIdByCrossIdAndPhaseNo(crossId, phaseNo);
                if (ObjectUtil.isNotEmpty(phaseId)) {
                    crossPhaseMapper.update(crossPhasePO);
                } else {
                    crossPhaseMapper.insertOne(crossPhasePO);
                }
            }
            List<CrossLightsPO> crossLightsList = schemePhaseLightsVO.getCrossLightsList();
            for (CrossLightsPO crossLightsPO : crossLightsList) {
                String crossId = crossLightsPO.getCrossId();
                String lightsNo = crossLightsPO.getLightsNo();
                CrossLightsPO crossLightsPO1 = crossLightsMapper.selectByCrossIdAndLedNum(crossId, lightsNo);
                if (ObjectUtil.isNotEmpty(crossLightsPO1)) {
                    crossLightsMapper.update(crossLightsPO);
                } else {
                    crossLightsMapper.insertOne(crossLightsPO);
                }
            }
            List<CrossPhaseLightsPO> crossPhaseLightsPOList = schemePhaseLightsVO.getCrossPhaseLightsPOList();
            for (CrossPhaseLightsPO crossPhaseLightsPO : crossPhaseLightsPOList) {
                Integer lightsId = crossPhaseLightsPO.getLightsId();
                Integer phaseId = crossPhaseLightsPO.getPhaseId();
                String crossId = crossPhaseLightsPO.getCrossId();
                Integer id = crossPhaseLightsMapper.selectIdByThreeIds(lightsId, phaseId, crossId);
                // 如果数据库中有记录则跳过，无记录则插入
                if (ObjectUtil.isEmpty(id)) {
                    crossPhaseLightsMapper.insertOne(crossPhaseLightsPO);
                }
            }
        }
        return jsonViewObject.success();
    }

    @Override
    public PhaseListByTimeVO phaseListByTime(CrossIdAndTimeDTO crossIdAndTimeDTO) {
        PhaseListByTimeVO phaseListByTimeVO = new PhaseListByTimeVO();
        String crossId = crossIdAndTimeDTO.getCrossId();
        Date datetime = crossIdAndTimeDTO.getDatetime();
        String dateStr = sdf.format(datetime);
        CrossIdDTO crossIdDTO = new CrossIdDTO();
        crossIdDTO.setCrossId(crossId);
        // 获取所有计划
        RunningPlanDTO runningPlanDTO = runningPlanServiceImpl.listRunningPlan(crossIdDTO);
        List<RunningPlanDTO.SchedulesPlanListElement> schedulesPlanList = runningPlanDTO.getSchedulesPlanList();
        for (RunningPlanDTO.SchedulesPlanListElement schedulesPlan : schedulesPlanList) {
            List<RunningPlanDTO.ExecListElement> execList = schedulesPlan.getExecList();
            for (RunningPlanDTO.ExecListElement execListElement : execList) {
                // 先查询特殊日期，再查询星期
                List<String> specialDateList = execListElement.getSpecialDateList();
                List<Integer> week = execListElement.getWeek();
                if (CollectionUtil.isNotEmpty(specialDateList)) {
                    for (String specialDate : specialDateList) {
                        String substring = specialDate.substring(0, 10);
                        if (Objects.equals(dateStr, substring)) {
                            String planName = execListElement.getPlanName();
                            phaseListByTimeVO = getPhaseListByTimeVO(planName, datetime, runningPlanDTO, crossIdDTO);
                        }
                    }
                } else if (CollectionUtil.isNotEmpty(week)) {
                    for (Integer day : week) {
                        // 1表示周日，2表示周一
                        int today = DateUtil.dayOfWeek(datetime);
                        today -= 1;
                        if (today == 0) {
                            today = 7;
                        }
                        if (day == today) {
                            String planName = execListElement.getPlanName();
                            phaseListByTimeVO = getPhaseListByTimeVO(planName, datetime, runningPlanDTO, crossIdDTO);
                        }
                    }
                }
            }
        }
        return phaseListByTimeVO;
    }

    private PhaseListByTimeVO getPhaseListByTimeVO(String planName, Date datetime, RunningPlanDTO runningPlanDTO,
                                                   CrossIdDTO crossIdDTO) {
        PhaseListByTimeVO phaseListByTimeVO = new PhaseListByTimeVO();
        List<RunningPlanDTO.TimeListElement> timeList = new ArrayList<>();
        List<RunningPlanDTO.DailyPlanListElement> dailyPlanList = runningPlanDTO.getDailyPlanList();
        for (RunningPlanDTO.DailyPlanListElement dailyPlan : dailyPlanList) {
            String name = dailyPlan.getName();
            if (Objects.equals(planName, name)) {
                timeList = dailyPlan.getTimeList();
            }
        }
        String hourMinuteStr = hourMinuteSdf.format(datetime);
        String hourMinuteStrReplace = hourMinuteStr.replace(":", "");
        int hourMinute = Integer.parseInt(hourMinuteStrReplace);
        String schemeName = null;
        for (RunningPlanDTO.TimeListElement timeListElement : timeList) {
            String startTimeStr = timeListElement.getStartTime();
            String startTimeStrReplace = startTimeStr.replace(":", "");
            int startTime = Integer.parseInt(startTimeStrReplace);
            String endTimeStr = timeListElement.getEndTime();
            String endTimeStrReplace = endTimeStr.replace(":", "");
            int endTime = Integer.parseInt(endTimeStrReplace);
            if (hourMinute >= startTime && hourMinute <= endTime) {
                String[] startEndTime = timeListElement.getStartEndTime();
                startEndTime[0] = startTimeStr;
                startEndTime[1] = endTimeStr;
                phaseListByTimeVO.setStartEndTime(startEndTime);
                schemeName = timeListElement.getSchemeName();
            }
        }
        // 查询方案
        SaveSchemeConfigDTO saveSchemeConfigDTO = schemeConfigServiceImpl.listSchemeConfig(crossIdDTO);
        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = saveSchemeConfigDTO.getPhaseSchemeList();
        for (SaveSchemeConfigDTO.PhaseScheme phaseScheme : phaseSchemeList) {
            String name = phaseScheme.getName();
            if (Objects.equals(name, schemeName)) {
                phaseListByTimeVO.setPhaseScheme(phaseScheme);
            }
        }
        return phaseListByTimeVO;
    }

    @NotNull
    private static ControlCommandVO getControlCommandVO(CrossIdDTO crossIdDTO) {
        ControlCommandVO commandVO = new ControlCommandVO();
        commandVO.setCrossCode(crossIdDTO.getCrossId());
        commandVO.setCommand(1); //控制标识：1是；0否
        commandVO.setDuration(0); //长期
        return commandVO;
    }

    private List<SchemeSendVO.Pattern> buildPatternList(String crossId, Integer schemeId) {
        List<SchemeSendVO.Pattern> patternList = new ArrayList<>();
        CrossSchemePO crossSchemePO = crossSchemeMapper.selectById(schemeId);
        SchemeSendVO.Pattern pattern = new SchemeSendVO.Pattern();
        pattern.setPatternNo(crossSchemePO.getSchemeNo());
        pattern.setPatternName(crossSchemePO.getName());
        pattern.setCycle(crossSchemePO.getCycle().toString());
        pattern.setCoordPhase(crossSchemePO.getCoordPhase().toString());
        pattern.setOffset(crossSchemePO.getOffset().toString());
        // 构造环列表
        List<SchemeSendVO.Pattern.Ring> rings = buildRings(crossId, schemeId);
        pattern.setRings(rings);
        patternList.add(pattern);

        return patternList;
    }

    private List<SchemeSendVO.Pattern.Ring> buildRings(String crossId, Integer schemeId) {
        List<SchemeSendVO.Pattern.Ring> rings = new ArrayList<>();
        // 根据路口ID和方案ID查询相位集合，根据ringNo分组，特殊控制ringNo为0
        List<CrossPhasePO> crossPhasePOList = crossPhaseMapper.selectByCrossIdAndSchemeId(crossId, schemeId);
        Map<Integer, List<CrossPhasePO>> collectMap = crossPhasePOList.stream()
                .collect(Collectors.groupingBy(CrossPhasePO::getRingNo));
        for (Map.Entry<Integer, List<CrossPhasePO>> entry : collectMap.entrySet()) {
            SchemeSendVO.Pattern.Ring ring = new SchemeSendVO.Pattern.Ring();
            ring.setRingNo(entry.getKey().toString());
            // 构造相位列表
            List<CrossPhasePO> phases = entry.getValue();
            List<SchemeSendVO.Pattern.Ring.Phase> phaseList = buildPhaseList(phases);
            ring.setPhaseList(phaseList);
            rings.add(ring);
        }
        return rings;
    }

    private List<SchemeSendVO.Pattern.Ring.Phase> buildPhaseList(List<CrossPhasePO> phases) {
        List<SchemeSendVO.Pattern.Ring.Phase> phaseList = new ArrayList<>();
        for (CrossPhasePO phasePO : phases) {
            SchemeSendVO.Pattern.Ring.Phase phase = new SchemeSendVO.Pattern.Ring.Phase();
            phase.setPhaseNo(phasePO.getPhaseNo());
            phase.setPhaseName(phasePO.getName());
            phase.setSort(phasePO.getSort().toString());
            phase.setControlMode(phasePO.getControlMode().toString());
            phase.setMinGreenTime(phasePO.getMinGreenTime().toString());
            phase.setMaxGreenTime(phasePO.getMaxGreenTime().toString());
            phase.setPhaseTime(phasePO.getPhaseTime().toString());
            phase.setGreenTime(phasePO.getGreenTime().toString());
            phase.setGreenFlashTime(phasePO.getGreenFlashTime().toString());
            phase.setPedFlashTime(phasePO.getPedFlashTime().toString());
            phase.setYellowTime(phasePO.getYellowTime().toString());
            phase.setRedTime(phasePO.getRedTime().toString());

            phaseList.add(phase);
        }
        return phaseList;
    }

    private List<ScheduleSendVO.Schedule> buildSchedules(String crossId, ScheduleIdDTO scheduleIdDTO) {
        List<ScheduleSendVO.Schedule> schedules = new ArrayList<>();
        Integer scheduleId = scheduleIdDTO.getScheduleId();
        List<CrossSchedulesPlanPO> crossSchedulesPlanPOList =
                crossSchedulesPlanMapper.selectByCrossIdAndSchedulesId(crossId, scheduleId);
        ScheduleSendVO.Schedule schedule = new ScheduleSendVO.Schedule();
        List<ScheduleSendVO.Schedule.Week> weeks = new ArrayList<>();
        List<ScheduleSendVO.Schedule.SpecialDay> specialDays = new ArrayList<>();
        for (CrossSchedulesPlanPO crossSchedulesPlanPO : crossSchedulesPlanPOList) {
            Integer planId = crossSchedulesPlanPO.getPlanId();
            Integer week = crossSchedulesPlanPO.getWeek();
            Date specialDate = crossSchedulesPlanPO.getSpecialDate();
            if (week != 0) {
                // 添加至星期列表
                addToWeeks(weeks, week, planId);
            } else {
                // 添加至特殊日期列表
                addToSpecialDays(specialDays, specialDate, planId);
            }
        }
        schedule.setWeeks(weeks);
        schedule.setSpecialDays(specialDays);
        schedules.add(schedule);
        return schedules;
    }

    private void addToSpecialDays(List<ScheduleSendVO.Schedule.SpecialDay> specialDays,
                                  Date specialDate, Integer planId) {
        ScheduleSendVO.Schedule.SpecialDay specialDay = new ScheduleSendVO.Schedule.SpecialDay();
        specialDay.setDateStr(sdf.format(specialDate));
        String planNo = getPlanNo(planId);
        specialDay.setPlanNo(Integer.parseInt(planNo));
        specialDays.add(specialDay);
    }

    private void addToWeeks(List<ScheduleSendVO.Schedule.Week> weeks, Integer week, Integer planId) {
        ScheduleSendVO.Schedule.Week weekObj = new ScheduleSendVO.Schedule.Week();
        weekObj.setWeekNum(week);
        String planNo = getPlanNo(planId);
        weekObj.setPlanNo(Integer.parseInt(planNo));
        weeks.add(weekObj);
    }

    private String getPlanNo(Integer planId) {
        CrossPlanPO crossPlanPO = crossPlanMapper.selectById(planId);
        return crossPlanPO.getPlanNo();
    }
}
